
LRLib_Serialise = {}

local function basicSerialize (o)
  if type(o) == "number" or type(o) == "boolean" then
    return tostring(o)
  else
    return string.format("%q", o)
  end
end

-- Lua keywords might look OK to not be quoted as keys but must be.
-- So, we make a list of them.

local lua_reserved_words = {}

for _, v in ipairs ({
    "and", "break", "do", "else", "elseif", "end", "false", 
    "for", "function", "if", "in", "local", "nil", "not", "or", 
    "repeat", "return", "then", "true", "until", "while"
            }) do
            
   lua_reserved_words [v] = true
end

-- Saves tables *without* cycles 

function save_item_simple (value, out)

  -- Numbers, strings, and booleans can be simply serialized

  if type (value) == "number" or 
     type (value) == "string" or
     type (value) == "boolean" then
     
     table.insert (out, basicSerialize(value))
      
  elseif type (value) == "table" then
  
    table.insert (out, "{")

    -- Save its fields
    for k, v in pairs (value) do   
       
      if not string.find (k, '^[_%a][_%a%d]*$') or lua_reserved_words[k] then
         table.insert (out, "[" .. basicSerialize (k) .. "] = ")
      else
         table.insert (out, k .. " = ")
      end
      
      save_item_simple (v, out)
         
      table.insert (out, ",")
      
    end 
      
    table.insert (out, "}")

  else
  
    error ("Cannot serialize " .. type (value))
    
  end 
  
end 

function LRLib_Serialise.serialise(v)
  local out = {}  -- output to this table
  save_item_simple (v, out)   -- do serialization
  return table.concat (out)   -- turn into a string 
end 