local LrApplication = import 'LrApplication'

require "Info"
require "PhotoShelterHelper"

return {
	metadataFieldsForPhotos = {
		{
			id = 'publishData',
			dataType = 'string',
		},
		{
			id = 'id',
			title = LOC("$$$/PhotoShelter/Metadata/ID=PhotoShelter ID"),
			dataType = 'string',
			readOnly = true,
			browsable = true,
			searchable = true,
		},
		{
			id = 'onServer',
			title = LOC("$$$/PhotoShelter/Metadata/OnPhotoShelter=On PhotoShelter"),
			dataType = 'enum',
			values = {
				{
					value = nil,
					title = LOC("$$$/CustomMetadata/Fields/False=False")
				},
				{
					value = true,
					title = LOC("$$$/CustomMetadata/Fields/True=True")
				},
			},
			readOnly = true,
			browsable = true,
			searchable = true,
		},
		{
			id = 'propRelease',
			title = LOC("$$$/PhotoShelter/Metadata/PropRelease=Property Released"),
			dataType = 'enum',
			values = {
				{
					value = nil,
					title = LOC("$$$/CustomMetadata/Fields/False=No")
				},
				{
					value = true,
					title = LOC("$$$/CustomMetadata/Fields/True=Yes")
				},
			},
			readOnly = true,
			browsable = true,
			searchable = true,
		},
		{
			id = 'modelRelease',
			title = LOC("$$$/PhotoShelter/Metadata/ModelRelease=Model Released"),
			dataType = 'enum',
			values = {
				{
					value = nil,
					title = LOC("$$$/CustomMetadata/Fields/False=No")
				},
				{
					value = true,
					title = LOC("$$$/CustomMetadata/Fields/True=Yes")
				},
			},
			readOnly = true,
			browsable = true,
			searchable = true,
		},
	},
	-- The PACT plugin was at version 2.
	schemaVersion = 3,
	updateFromEarlierSchemaVersion = function(catalog, previousSchemaVersion)
		-- Note: This function is called from within a catalog:withPrivateWriteAccessDo
		-- block. You should not call any of the with___Do functions yourself.
		if previousSchemaVersion == nil or previousSchemaVersion < 3 then
			local function initMetadata(publishService, parentSet)
				PhotoShelterHelper.recurseHierarchy(publishService, parentSet, function(publishedPhoto)
					local photoShelterID = publishedPhoto:getRemoteId()

					if photoShelterID then
						local suffix = photoShelterID:sub(-8)
						local lrPhoto = publishedPhoto:getPhoto()

						if suffix == "_removed" then
							photoShelterID = photoShelterID:sub(1, -8)
						end

						lrPhoto:setPropertyForPlugin(_PLUGIN, 'id', photoShelterID)
						lrPhoto:setPropertyForPlugin(_PLUGIN, 'onServer', true)
					end
				end)
			end

			-- Enumerate all the publish services and update metadata
			local catalog = LrApplication.activeCatalog()
			local publishedServices = catalog:getPublishServices(PhotoShelterInfo.namespace)

			if publishedServices and #publishedServices > 0 then
				for _, publishService in ipairs(publishedServices) do
					initMetadata(publishService)
				end
			end
		end
	end
}
